

import org.apache.commons.dbcp.*;
import org.apache.commons.pool.impl.*;


import java.util.*;
import java.io.*;
import java.sql.*;

/**
 * Основной класс. Он же пускалка. Хранит информацию об адресной книге, умеет добавлять в эту
 * книгу записи (а также их удалять и редактировать).
 */
public class AddressBook {
    private PoolingDataSource _poolingDataSource;

    /**
     * Новый конструктор, который создает пул соединений для дальнейшего использования.
     */
    public AddressBook() {
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }

        String connectionString = "jdbc:mysql://localhost:3306/addressboo?user=root&passowrd=";
        GenericObjectPool connectionPool = new GenericObjectPool(null);
        ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(connectionString, null);
        new PoolableConnectionFactory(connectionFactory, connectionPool, null, null, false, true);
        _poolingDataSource = new PoolingDataSource(connectionPool);
    }

    /**
     * Возвращает все контакты
     * @return
     */
    public SortedMap<String, Contact> getContacts() {
        try {
            SortedMap<String, Contact> result = new TreeMap<String, Contact>();

            Connection connection = _poolingDataSource.getConnection();

            try {
                PreparedStatement statement = connection.prepareStatement(
                        "SELECT `name`, `number`, `comment` FROM `addresses` WHERE 1");
                ResultSet resultSet = statement.executeQuery();

                while (resultSet.next()) {
                    Contact contact = new Contact();
                    contact.setData(resultSet.getString(1), resultSet.getString(2), resultSet.getString(3));
                    result.put(resultSet.getString(1), contact);
                }

                statement.close();
            } finally {
                connection.close();
            }

            return result;
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return new TreeMap<String, Contact>();
    }

    /**
     * Ищет и возвращает контакт по имени
     * @param aNumber
     * @return
     */
    public Contact getContactByNumber(String aNumber) {
        try {
            Contact result = null;

            Connection connection = _poolingDataSource.getConnection();

            try {
                PreparedStatement statement = connection.prepareStatement(
                        "SELECT `name`, `number`, `comment` FROM `addresses` WHERE `number`=?");
                statement.setString(1, aNumber);
                ResultSet resultSet = statement.executeQuery();

                if (resultSet.next()) {
                    result = new Contact();
                    result.setData(resultSet.getString(1), resultSet.getString(2), resultSet.getString(3));
                }

                statement.close();
            } finally {
                connection.close();
            }

            return result;
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Добавляет контакт в адресную книгу
     *
     * @param aName    Имя
     * @param aNumber  Телефонный номер
     * @param aComment Комментарий
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void addContact(String aName, String aNumber, String aComment) throws IOException {
        try {
            Connection connection = _poolingDataSource.getConnection();

            try {
                PreparedStatement statement = connection.prepareStatement(
                        "INSERT INTO `addresses` (`name`, `number`, `comment`) VALUES (?, ?, ?)");
                statement.setString(1, aName);
                statement.setString(2, aNumber);
                statement.setString(3, aComment);
                statement.executeUpdate();
                statement.close();
            } finally {
                connection.close();
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /**
     * Удаляет контакт по телефонному номеру.
     *
     * @param aNumber телефонный номер, который определяет удаляемый контакт.
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void removeContactByNumber(String aNumber) throws IOException {
        try {
            Connection connection = _poolingDataSource.getConnection();

            try {
                PreparedStatement statement = connection.prepareStatement("DELETE FROM `addresses` WHERE `number`=?");
                statement.setString(1, aNumber);
                statement.executeUpdate();
                statement.close();
            } finally {
                connection.close();
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /**
     * Редактирует контакт, если такой контакт существует.
     *
     * @param aEditNumber Номер, который определяет редактируемый контакт.
     * @param aName       Новое имя контакта.
     * @param aNumber     Новый номер контакта.
     * @param aComment    Новый комментарий для контакта.
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void editContactByNumber(String aEditNumber, String aName, String aNumber, String aComment) throws IOException {
        try {
            Connection connection = _poolingDataSource.getConnection();

            try {
                PreparedStatement statement = connection.prepareStatement(
                        "UPDATE `addresses` SET `name`=?, `number`=?, `comment`=? WHERE `number`=?");
                statement.setString(1, aName);
                statement.setString(2, aNumber);
                statement.setString(3, aComment);
                statement.setString(4, aEditNumber);
                statement.executeUpdate();
                statement.close();
            } finally {
                connection.close();
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}